@@ -35,9 +35,11 @@ __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
 from . import models
+from . import partials
 from . import social
 from . import part
 
 from .models import *
+from .partials import *
 from .social import *
 from .part import AdminPart
