@@ -34,6 +34,7 @@ __copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
+from . import oauth
 from . import account
 from . import base
 from . import config
@@ -43,6 +44,7 @@ from . import search
 from . import settings
 from . import snapshot
 
+from .oauth import *
 from .account import Account
 from .base import Base
 from .config import Config
