@@ -44,8 +44,9 @@ import binascii
 
 from appier_extras.parts.admin.models import base
 from appier_extras.parts.admin.models import role
+from appier_extras.parts.admin.partials import authenticable
 
-class Account(base.Base):
+class Account(base.Base, authenticable.Authenticable):
 
     ADMIN_TYPE = 1
     USER_TYPE = 2
@@ -112,9 +113,10 @@ class Account(base.Base):
         description = "Facebook ID"
     )
 
-    twitter_username = appier.field(
+    github_login = appier.field(
         index = "hashed",
-        safe = True
+        safe = True,
+        description = "GitHub Login"
     )
 
     google_id = appier.field(
@@ -129,7 +131,7 @@ class Account(base.Base):
         description = "Live ID"
     )
 
-    github_login = appier.field(
+    twitter_username = appier.field(
         index = "hashed",
         safe = True
     )
@@ -139,7 +141,7 @@ class Account(base.Base):
         meta = "secret"
     )
 
-    twitter_token = appier.field(
+    github_token = appier.field(
         private = True,
         meta = "secret"
     )
@@ -154,7 +156,7 @@ class Account(base.Base):
         meta = "secret"
     )
 
-    github_token = appier.field(
+    twitter_token = appier.field(
         private = True,
         meta = "secret"
     )
@@ -498,6 +500,7 @@ class Account(base.Base):
 
     @classmethod
     def _build(cls, model, map):
+        super(Account, cls)._build(model, map)
         username = model["username"]
         model["avatar_url"] = cls._get_avatar_url_g(username)
 
@@ -506,6 +509,7 @@ class Account(base.Base):
         prefixes = prefixes or cls.PREFIXES
         session = appier.get_session()
         delete = getattr(session, method)
+        if "cls" in session: delete("cls")
         if "username" in session: delete("username")
         if "name" in session: delete("name")
         if "email" in session: delete("email")
@@ -530,12 +534,20 @@ class Account(base.Base):
     def _get_avatar_url_g(cls, username, absolute = True, owner = None):
         owner = owner or appier.get_app()
         if not hasattr(owner, "admin_part"): return None
+        model = None if cls._is_master() else cls._name()
         return owner.url_for(
             "admin.avatar_account",
             username = username,
+            cls = model,
             absolute = absolute
         )
 
+    @classmethod
+    def _is_master(cls, owner = None):
+        owner = owner or appier.get_app()
+        admin_part = owner.admin_part
+        return cls == admin_part.account_c
+
     def pre_validate(self):
         base.Base.pre_validate(self)
         if hasattr(self, "username") and self.username:
@@ -598,6 +610,19 @@ class Account(base.Base):
         self.save()
 
     def tokens(self):
+        """
+        Retrieves the complete set of ACL tokens for the current use
+        respecting the wild card based selection if there's such use.
+
+        These should consider both dynamic and static role users.
+
+        This method has a critical impact on overall system security.
+
+        :rtype: List
+        :return: The ACL tokens list taking into consideration the complete
+        aggregated set of roles of the user.
+        """
+
         tokens = set()
 
         if self.type == Account.ADMIN_TYPE:
@@ -616,7 +641,19 @@ class Account(base.Base):
 
         return tokens
 
-    def views(self):
+    def views_l(self):
+        """
+        Retrieves the complete set of views for the current account,
+        these should take into consideration the account's roles.
+
+        Aggregation of the multiple views should be done in a linear
+        fashion with the first being the most relevant one.
+
+        :rtype: List
+        :return: The list of view maps that should be applied for proper
+        context filtering of the user (result set constrain).
+        """
+
         views = []
         for role in self.roles_l:
             view_m = role.view_m(context = self)
@@ -633,6 +670,25 @@ class Account(base.Base):
         cls = self.__class__
         return cls.generate(value)
 
+    def _send_avatar(self, image = "avatar.png", strict = False, cache = False):
+        admin_part = self.owner.admin_part
+        avatar = self.avatar if hasattr(self, "avatar") else None
+        if not avatar:
+            if strict: raise appier.NotFoundError(
+                message = "Avatar not found for user '%s'" % self.username,
+                code = 404
+            )
+            return self.owner.send_static(
+                "images/" + image,
+                static_path = admin_part.static_path
+            )
+        return self.owner.send_file(
+            avatar.data,
+            content_type = avatar.mime,
+            etag = avatar.etag,
+            cache = cache
+        )
+
     def _set_avatar_d(self, image = "avatar.png", mime = "image/png"):
         if not hasattr(self.owner, "admin_part"): return
         if not self.owner.admin_part: return
@@ -668,20 +724,20 @@ class Account(base.Base):
         self.last_login = time.time()
         self.save()
 
-    @appier.operation(name = "Generate Key")
+    @appier.operation(name = "Generate Key", level = 2)
     def generate_key_s(self, force = False):
         self = self.reload(rules = False)
         if self.key and not force: return
         self.key = self.secret()
         self.save()
 
-    @appier.operation(name = "Mark Unconfirmed")
+    @appier.operation(name = "Mark Unconfirmed", level = 2)
     def mark_unconfirmed_s(self):
         self.enabled = False
         self.confirmation_token = self.secret()
         self.save()
 
-    @appier.operation(name = "Email New")
+    @appier.operation(name = "Email New", level = 2)
     def email_new(self, password = None):
         account = self.reload(rules = False, meta = True)
         base.Base.send_email_g(
@@ -694,7 +750,7 @@ class Account(base.Base):
             account_password = password
         )
 
-    @appier.operation(name = "Email Confirm")
+    @appier.operation(name = "Email Confirm", level = 2)
     def email_confirm(self):
         account = self.reload(rules = False, meta = True)
         base.Base.send_email_g(
@@ -706,7 +762,7 @@ class Account(base.Base):
             account = account
         )
 
-    @appier.operation(name = "Email Recover")
+    @appier.operation(name = "Email Recover", level = 2)
     def email_recover(self):
         account = self.reload(rules = False, meta = True)
         base.Base.send_email_g(
@@ -718,11 +774,24 @@ class Account(base.Base):
             account = account
         )
 
+    @appier.operation(
+        name = "Upload Avatar",
+        parameters = (("Avatar", "avatar", "file"),)
+    )
+    def upload_avatar_s(self, avatar):
+        cls = self.__class__
+        if not avatar: return
+        self.avatar = cls.avatar.type(avatar)
+        self.save()
+
     @appier.link(name = "View Avatar", devel = True)
     def view_avatar_url(self, absolute = False):
-        return appier.get_app().url_for(
+        cls = self.__class__
+        model = None if cls._is_master() else cls._name()
+        return self.owner.url_for(
             "admin.avatar_account",
             username = self.username,
+            cls = model,
             absolute = absolute
         )
 
@@ -736,14 +805,15 @@ class Account(base.Base):
 
     def _set_session(self, unset = True, safes = [], method = "set"):
         cls = self.__class__
-        if unset: cls._unset_session(safes = safes)
+        if unset: cls._unset_account(safes = safes)
         self.session.ensure()
         set = getattr(self.session, method)
+        set("cls", cls.__name__)
         set("username", self.username)
         set("name", self.email)
         set("email", self.email)
         set("type", self.type_s())
         set("tokens", self.tokens())
-        set("views", self.views())
+        set("views", self.views_l())
         set("meta", self.meta)
         set("params", dict())
