@@ -53,6 +53,14 @@ going to be used in the generation of the random string
 value that is going to be used as basis for secret generation """
 
 class Base(appier.Model):
+    """
+    Top level abstract class from which all the appier admin
+    based models should inherit. It contains utility structures
+    and logic to better handle a model under appier admin.
+
+    Only models that inherit from this abstract model will be
+    shown in the appier admin back-office.
+    """
 
     ENABLE_S = {
         True : "enabled",
@@ -64,7 +72,9 @@ class Base(appier.Model):
         index = "all",
         increment = True,
         safe = True,
-        description = "ID"
+        description = "ID",
+        observations = """The base identifier of the entity to
+        be used as a unique value on the entity context"""
     )
     """ The global incremental identifier of the base model
     this should always be considered a unique way of addressing
@@ -75,7 +85,10 @@ class Base(appier.Model):
         index = "all",
         initial = True,
         meta = "enum",
-        enum = ENABLE_S
+        enum = ENABLE_S,
+        observations = """Simple flag that controls if the entity
+        is considered to be enabled, respecting this flag is not
+        mandatory and relies on specific usage context"""
     )
     """ Boolean field that defines if a certain entity is considered
     to be enabled or disabled, the meaning of disable should depend
@@ -84,7 +97,9 @@ class Base(appier.Model):
 
     description = appier.field(
         meta = "text",
-        default = True
+        default = True,
+        observations = """Plain text description of the entity, proper
+        usage should be defined by the context"""
     )
     """ Global description value for which the meaning should be defined
     by the context of usage (eg: name, observations, etc.) """
@@ -94,7 +109,9 @@ class Base(appier.Model):
         index = "all",
         safe = True,
         immutable = True,
-        meta = "datetime"
+        meta = "datetime",
+        observations = """Date when the entity was initially created, should
+        be immutable (not changeable)"""
     )
     """ The original date and time of the entity creation, this should
     be an immutable timestamp and never changed """
@@ -103,14 +120,17 @@ class Base(appier.Model):
         type = int,
         index = "all",
         safe = True,
-        meta = "datetime"
+        meta = "datetime",
+        observations = """Date when the entity was last modified"""
     )
     """ The date and time of the latest save operation for the current
     entity, note that the create operation changes this value """
 
     meta = appier.field(
         type = dict,
-        description = "Metadata"
+        description = "Metadata",
+        observations = """Dictionary based information to be "attached"
+        to the entity, proper usage depends on context"""
     )
     """ Additional (unstructured) information to be stored together with
     the entity for unpredicted purposes, note that these values should not
@@ -190,7 +210,7 @@ class Base(appier.Model):
         return names
 
     @classmethod
-    def order_name(self):
+    def order_name(cls):
         return "id"
 
     @classmethod
@@ -205,6 +225,10 @@ class Base(appier.Model):
     def description_name(cls):
         return None
 
+    @classmethod
+    def is_abstract(cls):
+        return True
+
     @classmethod
     def is_indexed(cls):
         return True
@@ -245,7 +269,6 @@ class Base(appier.Model):
         unsubscribe = kwargs.pop("unsubscribe", unsubscribe)
         logo = kwargs.pop("logo", logo)
         inline = kwargs.pop("inline", inline)
-        kwargs["owner"] = owner
         settings = dict(
             bulk = bulk,
             unsubscribe = unsubscribe,
@@ -255,7 +278,7 @@ class Base(appier.Model):
         if bulk: headers["Auto-Submitted"] = "auto-generated"
         if bulk: headers["Precedence"] = "bulk"
         if unsubscribe: headers["List-Unsubscribe"] = "<" + base_url + "/unsubscribe>"
-        html_handler = lambda html: cls._inlinify(html)
+        html_handler = cls._inlinify
         html_handler = html_handler if inline else None
         owner.email(
             sender = sender,
@@ -304,10 +327,6 @@ class Base(appier.Model):
     def op_empty_s(cls):
         cls.delete_c()
 
-    @classmethod
-    def _build(cls, model, map):
-        pass
-
     @classmethod
     def _csv_import(
         cls,
@@ -329,6 +348,7 @@ class Base(appier.Model):
             )
         args, _varargs, kwargs = appier.legacy.getargspec(callback)[:3]
         has_header = True if "header" in args or kwargs else False
+        has_map = True if "map" in args or kwargs else False
         if is_unicode:
             data = data.decode(encoding)
             buffer = appier.legacy.StringIO(data)
@@ -343,10 +363,12 @@ class Base(appier.Model):
         header = next(csv_reader)
         if named: tuple_t = collections.namedtuple("csv_tuple", header)
         for line in csv_reader:
+            kwargs = dict()
             if not is_unicode: line = [value.decode(encoding) for value in line]
-            if named: line = tuple_t(line)
-            if has_header: callback(line, header = header)
-            else: callback(line)
+            if named: line = tuple_t(*line)
+            if has_header: kwargs["header"] = header
+            if has_map: kwargs["map"] = dict(zip(header, line))
+            callback(line, **kwargs)
 
     @classmethod
     def _inlinify(cls, data, engine = None):
@@ -393,6 +415,22 @@ class Base(appier.Model):
 
         self.destroy_index()
 
+    def previous(self, name = "id", raise_e = False, *args, **kwargs):
+        kwargs[name] = {"$lt" : getattr(self, name)}
+        kwargs["sort"] = ((name, -1),)
+        return self.get_v(
+            raise_e = raise_e,
+            **kwargs
+        )
+
+    def next(self, name = "id", raise_e = False, *args, **kwargs):
+        kwargs[name] = {"$gt" : getattr(self, name)}
+        kwargs["sort"] = ((name, 1),)
+        return self.get_v(
+            raise_e = raise_e,
+            **kwargs
+        )
+
     def build_index(self, use_class = True):
         from appier_extras.parts.admin.models import search
 
@@ -485,9 +523,6 @@ class Base(appier.Model):
         # the provided model data should always represent the complete state
         snapshot.Snapshot.create_snapshot(self.id, cls, model_data = model)
 
-    def touch_s(self):
-        self.save()
-
     def enable_s(self):
         self.enabled = True
         self.save()
@@ -496,6 +531,9 @@ class Base(appier.Model):
         self.enabled = False
         self.save()
 
+    def touch_s(self):
+        self.save()
+
     def to_locale(self, *args, **kwargs):
         return self.owner.to_locale(*args, **kwargs)
 
@@ -509,11 +547,6 @@ class Base(appier.Model):
         url_sha1 = hashlib.sha1(token_bytes)
         return url_sha1.hexdigest()
 
-    @appier.operation(name = "Touch")
-    def op_touch_s(self):
-        self = self.reload(rules = False)
-        self.touch_s()
-
     @appier.operation(name = "Enable")
     def op_enable_s(self):
         self = self.reload(rules = False)
@@ -524,6 +557,11 @@ class Base(appier.Model):
         self = self.reload(rules = False)
         self.disable_s()
 
+    @appier.operation(name = "Touch", devel = True)
+    def op_touch_s(self):
+        self = self.reload(rules = False)
+        self.touch_s()
+
     @property
     def created_d(self):
         return datetime.datetime.fromtimestamp(self.created)
