@@ -71,7 +71,17 @@ class Event(base.Base):
 
     @classmethod
     def list_names(cls):
-        return ["name", "handler"]
+        return ["name", "handler", "enabled"]
+
+    @classmethod
+    def transform(cls, arguments, prefix):
+        arguments = dict(arguments)
+        prefix_l = len(prefix)
+        for key, value in appier.legacy.items(arguments):
+            if not key.startswith(prefix): continue
+            key_prefix = key[prefix_l:]
+            arguments[key_prefix] = value
+        return arguments
 
     @classmethod
     def notify_g(cls, name, handlers = None, arguments = {}):
@@ -79,24 +89,25 @@ class Event(base.Base):
         logger.debug("Triggering '%s' event ..." % name)
         kwargs = dict(name = name)
         if handlers: kwargs["handler"] = {"$in" : handlers}
-        events = cls.find(**kwargs)
+        events = cls.find_e(**kwargs)
         for event in events: event.notify(arguments = arguments)
 
     @appier.operation(name = "Notify")
-    def notify(self, arguments = {}, delay = True, owner = None):
+    def notify(self, arguments = {}, delay = True):
+        cls = self.__class__
         delay_s = ("a delayed" if delay else "an immediate")
         logger = appier.get_logger()
         logger.debug(
             "Notifying handler '%s' for '%s' in %s fashion ..." %\
             (self.handler, self.name, delay_s)
         )
-        owner = owner or appier.get_app()
         method = getattr(self, "notify_" + self.handler)
         arguments_m = dict(self.arguments)
         arguments_m.update(arguments)
         arguments_m.update(event = self.name, handler = self.handler)
+        arguments_m = cls.transform(arguments_m, self.handler + "_")
         kwargs = dict(arguments = arguments_m)
-        if delay: owner.delay(method, kwargs = kwargs)
+        if delay: self.owner.delay(method, kwargs = kwargs)
         else: method(arguments, **kwargs)
 
     @classmethod
@@ -139,12 +150,24 @@ class Event(base.Base):
         return appier.post(url, data_j = arguments)
 
     def notify_mailme(self, arguments = {}):
+        appier.ensure_pip("mailme", package = "mailme_api")
         import mailme
         logger = appier.get_logger()
         logger.debug("Running Mailme notification ...")
         api = mailme.Api()
         return api.send(arguments)
 
+    def notify_nexmo(self, arguments = {}):
+        appier.ensure_pip("nexmo", package = "nexmo_api")
+        import nexmo
+        sender = arguments["sender"]
+        receiver = arguments["receiver"]
+        text = arguments["text"]
+        logger = appier.get_logger()
+        logger.debug("Running Nexmo notification for '%s' ..." % receiver)
+        api = nexmo.Api()
+        return api.send_sms(sender, receiver, text)
+
     @appier.operation(name = "Duplicate", factory = True)
     def duplicate_s(self):
         cls = self.__class__
