@@ -78,6 +78,7 @@ class AdminPart(
         self.theme = appier.conf("ADMIN_THEME", "flat")
         self.style = appier.conf("ADMIN_STYLE", "")
         self.libs = appier.conf("ADMIN_LIBS", "current")
+        self.social_libs = appier.conf("ADMIN_SOCIAL_LIBS", [], cast = list)
 
     def load(self):
         appier.Part.load(self)
@@ -106,6 +107,9 @@ class AdminPart(
         self.owner.admin_open = appier.conf(
             "ADMIN_OPEN", False, cast = bool
         )
+        self.owner.admin_oauth = appier.conf(
+            "ADMIN_OAUTH", True, cast = bool
+        )
         self.owner.admin_avatar_default = appier.conf(
             "ADMIN_AVATAR_DEFAULT", False, cast = bool
         )
@@ -126,6 +130,10 @@ class AdminPart(
             if not model_c.is_concrete(): continue
             self.logger.debug(model_c)
 
+        for social_lib in self.social_libs:
+            method = getattr(self, "ensure_" + social_lib)
+            method()
+
     def routes(self):
         return [
             (("GET",), "/admin", self.index),
@@ -146,6 +154,11 @@ class AdminPart(
             (("GET",), "/admin/configs", self.list_configs),
             (("GET",), "/admin/parts", self.list_parts),
             (("GET",), "/admin/libraries", self.list_libraries),
+            (("GET",), "/admin/oauth/authorize", self.oauth_authorize),
+            (("POST",), "/admin/oauth/authorize", self.do_oauth_authorize),
+            (("GET",), "/admin/oauth/deny", self.oauth_deny),
+            (("GET", "POST"), "/admin/oauth/access_token", self.oauth_access_token, None, True),
+            (("GET", "POST"), "/admin/oauth/login", self.oauth_login, None, True),
             (("GET",), "/admin/operations/build_index", self.build_index),
             (("GET",), "/admin/operations/build_index_db", self.build_index_db),
             (("GET",), "/admin/operations/test_email", self.test_email),
@@ -166,6 +179,7 @@ class AdminPart(
             (("GET",), "/admin/search", self.search),
             (("GET",), "/admin/accounts/new", self.new_account),
             (("POST",), "/admin/accounts", self.create_account),
+            (("GET",), "/admin/accounts/me", self.me_account),
             (("GET",), "/admin/accounts/<str:username>", self.show_account),
             (("GET",), "/admin/accounts/<str:username>/mail", self.mail_account),
             (("GET",), "/admin/accounts/<str:username>/avatar", self.avatar_account),
@@ -175,6 +189,7 @@ class AdminPart(
             (("GET",), "/admin/models/<str:model>", self.show_model),
             (("GET", "POST"), "/admin/models/<str:model>/links/<str:link>", self.link_model),
             (("GET", "POST"), "/admin/models/<str:model>/operations/<str:operation>", self.operation_model),
+            (("GET", "POST"), "/admin/models/<str:model>/view/<str:view>", self.view_model),
             (("GET",), "/admin/models/<str:model>/new", self.new_entity),
             (("POST",), "/admin/models/<str:model>", self.create_entity),
             (("GET",), "/admin/models/<str:model>/<str:_id>.json", self.show_entity_json, None, True),
@@ -184,23 +199,29 @@ class AdminPart(
             (("GET",), "/admin/models/<str:model>/<str:_id>/delete", self.delete_entity),
             (("GET",), "/admin/facebook", self.facebook),
             (("GET",), "/admin/facebook/unlink", self.unlink_facebook),
+            (("GET",), "/admin/facebook/unset", self.unset_facebook),
             (("GET",), "/admin/facebook/oauth", self.oauth_facebook),
             (("GET",), "/admin/github", self.github),
             (("GET",), "/admin/github/unlink", self.unlink_github),
+            (("GET",), "/admin/github/unset", self.unset_github),
             (("GET",), "/admin/github/oauth", self.oauth_github),
             (("GET",), "/admin/google", self.google),
             (("GET",), "/admin/google/unlink", self.unlink_google),
+            (("GET",), "/admin/google/unset", self.unset_google),
             (("GET",), "/admin/google/oauth", self.oauth_google),
             (("GET",), "/admin/live", self.live),
             (("GET",), "/admin/live/unlink", self.unlink_live),
+            (("GET",), "/admin/live/unset", self.unset_live),
             (("GET",), "/admin/live/oauth", self.oauth_live),
             (("GET",), "/admin/twitter", self.twitter),
             (("GET",), "/admin/twitter/unlink", self.unlink_twitter),
+            (("GET",), "/admin/twitter/unset", self.unset_twitter),
             (("GET",), "/admin/twitter/oauth", self.oauth_twitter),
             (("GET",), "/admin/log.json", self.show_log, None, True),
             (("GET",), "/admin/configs.json", self.show_configs, None, True),
             (("GET",), "/api/admin/ping", self.ping_api, None, True),
             (("GET", "POST"), "/api/admin/login", self.login_api, None, True),
+            (("GET",), "/api/admin/accounts/me", self.me_account_api, None, True),
             (("GET",), "/api/admin/models/<str:model>", self.show_model_json, None, True),
             (("GET",), "/api/admin/models/<str:model>/<str:_id>", self.show_entity_json, None, True)
         ]
@@ -214,7 +235,6 @@ class AdminPart(
         return appier.Part.template(
             self,
             template,
-            owner = self.owner,
             models_d = self.models_d,
             *args,
             **kwargs
@@ -232,7 +252,7 @@ class AdminPart(
     def exception_handler(self, error):
         import traceback
         lines = traceback.format_exc().splitlines()
-        lines = self._lines(lines)
+        lines = appier.App._lines(lines)
         lines = lines if self.owner.is_devel() else []
         return self.template(
             "error.html.tpl",
@@ -290,7 +310,7 @@ class AdminPart(
         # verifies the existence of the various account related session
         # attributes and in case they exist removes them from session as
         # the user is currently logging out from session
-        self.account_c._unset_session()
+        self.account_c._unset_account()
 
         # runs the proper redirect operation, taking into account if the
         # next value has been provided or not
@@ -404,8 +424,24 @@ class AdminPart(
             )
         )
 
+    def me_account(self):
+        account_c = self._get_cls(self.account_c)
+        account = account_c.from_session(meta = True)
+        return self.template(
+            "account/show.html.tpl",
+            account = account
+        )
+
     def show_account(self, username):
-        raise appier.NotImplementedError()
+        account_c = self._get_cls(self.account_c)
+        account = account_c.get(
+            username = username,
+            meta = True
+        )
+        return self.template(
+            "account/show.html.tpl",
+            account = account
+        )
 
     def mail_account(self, username):
         raise appier.NotImplementedError()
@@ -413,24 +449,13 @@ class AdminPart(
     def avatar_account(self, username):
         strict = self.field("strict", False, cast = bool)
         cache = self.field("cache", False, cast = bool)
-        account = self.account_c.get(
+        account_c = self._get_cls(self.account_c)
+        account = account_c.get(
             username = username,
             rules = False
         )
-        avatar = account.avatar if hasattr(account, "avatar") else None
-        if not avatar:
-            if strict: raise appier.NotFoundError(
-                message = "Avatar not found for user '%s'" % username,
-                code = 404
-            )
-            return self.send_static(
-                "images/avatar.png",
-                static_path = self.static_path
-            )
-        return self.send_file(
-            avatar.data,
-            content_type = avatar.mime,
-            etag = avatar.etag,
+        return account._send_avatar(
+            strict = strict,
             cache = cache
         )
 
@@ -525,8 +550,7 @@ class AdminPart(
 
     @appier.ensure(token = "admin.status")
     def list_parts(self):
-        app = appier.get_app()
-        parts = app.get_parts()
+        parts = self.owner.get_parts()
         return self.template(
             "parts.html.tpl",
             section = "status",
@@ -535,14 +559,197 @@ class AdminPart(
 
     @appier.ensure(token = "admin.status")
     def list_libraries(self):
-        app = appier.get_app()
-        libraries = app.get_libraries()
+        libraries = self.owner.get_libraries()
         return self.template(
             "libraries.html.tpl",
             section = "status",
             libraries = libraries
         )
 
+    @appier.private
+    def oauth_authorize(self):
+        # verifies if the oauth system is allowed if that's not
+        # the case raises an exception indicating so
+        if not self.owner.admin_oauth: raise appier.SecurityError(
+            message = "OAuth not allowed"
+        )
+
+        # retrieves the complete set of fields that are going
+        # to be used on the initial authorize state of oauth
+        client_id = self.field("client_id", mandatory = True)
+        redirect_uri = self.field("redirect_uri", mandatory = True)
+        scope = self.field("scope", mandatory = True)
+        response_type = self.field("response_type", "code")
+        state = self.field("state", None)
+
+        # verifies/ensures that the response type to be received
+        # is the code, as that's the only one supported
+        appier.verify("response_type", "code")
+
+        # normalizes the scope value to the list representation
+        # and then sorts it in the normal manner (normalization)
+        scope_l = scope.split(" ")
+        scope_l.sort()
+
+        # runs the mandatory retrieval of the oauth client associated
+        # with the provided client id
+        oauth_client = models.OAuthClient.get_e(
+            client_id = client_id,
+            redirect_uri = redirect_uri
+        )
+
+        # tries to re-use an already authorized token that is considered
+        # equivalent to the current one, if the re-usage operation is a
+        # success then redirects the user agent immediately
+        result, tokens, oauth_token = models.OAuthToken.reuse_s(
+            redirect_uri, scope_l, oauth_client
+        )
+        if result: return self.redirect(
+            redirect_uri,
+            params = dict(
+                code = oauth_token.authorization_code,
+                scope = " ".join(oauth_token.tokens),
+                tokens = oauth_token.tokens,
+            )
+        )
+
+        # runs the template rendering for the oauth authorize panel
+        # it should prompt the final user for permission agreement
+        return self.template(
+            "oauth/authorize.html.tpl",
+            client_id = client_id,
+            redirect_uri = redirect_uri,
+            scope = scope,
+            response_type = response_type,
+            state = state,
+            oauth_client = oauth_client,
+            tokens = tokens
+        )
+
+    @appier.ensure()
+    def do_oauth_authorize(self):
+        if not self.owner.admin_oauth: raise appier.SecurityError(
+            message = "OAuth not allowed"
+        )
+
+        client_id = self.field("client_id", mandatory = True)
+        redirect_uri = self.field("redirect_uri", mandatory = True)
+        scope = self.field("scope", mandatory = True)
+        state = self.field("state", mandatory = True)
+
+        scope_l = scope.split(" ")
+        scope_l.sort()
+
+        account = self.account_c.from_session()
+        oauth_client = models.OAuthClient.get_e(
+            client_id = client_id,
+            redirect_uri = redirect_uri
+        )
+        oauth_token = oauth_client.build_token_s(
+            username = account.username,
+            scope = scope_l
+        )
+
+        return self.redirect(
+            redirect_uri,
+            params = dict(
+                code = oauth_token.authorization_code,
+                scope = " ".join(oauth_token.tokens),
+                state = state,
+            )
+        )
+
+    @appier.private
+    def oauth_deny(self):
+        if not self.owner.admin_oauth: raise appier.SecurityError(
+            message = "OAuth not allowed"
+        )
+        redirect_uri = self.field("redirect_uri", mandatory = True)
+        return self.redirect(
+            redirect_uri,
+            params = dict(
+                error = "access_denied",
+                error_description = "Permissions error"
+            )
+        )
+
+    def oauth_access_token(self):
+        # verifies if the oauth system is allowed if that's not
+        # the case raises an exception indicating so
+        if not self.owner.admin_oauth: raise appier.SecurityError(
+            message = "OAuth not allowed"
+        )
+
+        # retrieve the multiple fields that are going to be used for the
+        # process of issuing the access token (only authorization code is
+        # going to be returned to the client)
+        client_id = self.field("client_id", mandatory = True)
+        client_secret = self.field("client_secret", mandatory = True)
+        redirect_uri = self.field("redirect_uri", mandatory = True)
+        code = self.field("code", mandatory = True)
+        grant_type = self.field("grant_type", "authorization_code")
+
+        # tries to retrieve the oauth client associated with the
+        # provided client id and secret and then uses the value to
+        # retrieve the associated oauth token via association
+        oauth_client = models.OAuthClient.get_e(
+            client_id = client_id,
+            client_secret = client_secret,
+            redirect_uri = redirect_uri
+        )
+        oauth_token = models.OAuthToken.get_e(
+            authorization_code = code,
+            client = oauth_client.id,
+            rules = False
+        )
+
+        # verifies that the authorization code is the expected
+        # one and then unsets it from the oauth token, so that
+        # it's no longer going to be used
+        oauth_token.verify_code(code, grant_type = grant_type)
+        oauth_token.unset_code_s()
+
+        # returns the final map based response containing the complete
+        # set of elements to the client to be used
+        return dict(
+            access_token = oauth_token.access_token,
+            token_type = "bearer",
+            expires_in = oauth_token.expires_in,
+            refresh_token = oauth_token.refresh_token
+        )
+
+    def oauth_login(self):
+        # verifies if the oauth system is allowed if that's not
+        # the case raises an exception indicating so
+        if not self.owner.admin_oauth: raise appier.SecurityError(
+            message = "OAuth not allowed"
+        )
+
+        # retrieves the reference to the access token that has been
+        # provided to the request and then uses it to retrieve the
+        # token, note that an exception is raised if no access token
+        # is provided (as expected)
+        access_token = self.field("access_token", mandatory = True)
+        oauth_token = models.OAuthToken.login(access_token)
+
+        # updates the current session with the proper
+        # values to correctly authenticate the user
+        oauth_token._set_session()
+
+        # retrieves the session identifier (sid) for the currently
+        # assigned session, this is going to be used in the next
+        # requests to refer to the proper session
+        sid = self.session.sid
+
+        # redirects the current operation to the next url or in
+        # alternative to the root index of the administration
+        return dict(
+            sid = sid,
+            session_id = sid,
+            username = oauth_token.username,
+            tokens = oauth_token.tokens
+        )
+
     @appier.ensure(token = "admin")
     def build_index(self):
         empty = self.field("empty", True, cast = bool)
@@ -577,7 +784,7 @@ class AdminPart(
             message = "No test email defined"
         )
         models.Base.send_email_g(
-            appier.get_app(),
+            self.owner,
             "admin/email/test.html.tpl",
             receivers = [receiver],
             subject = "Test email",
@@ -805,8 +1012,8 @@ class AdminPart(
             page = True,
             find = True
         )
-        page = model.paginate_v(**object)
         object = self._sort(object, model)
+        page = model.paginate_v(**object)
         entities = model.find_v(meta = True, **object)
         return self.template(
             "models/show.html.tpl",
@@ -850,24 +1057,68 @@ class AdminPart(
 
     @appier.ensure(token = "admin")
     def link_model(self, model, link):
+        # ensures that the proper token is available so that the
+        # operation may be executed under the current context
+        appier.ensure_login(self, token = "admin.models." + model)
+
+        # retrieves the complete set of sequential parameters that
+        # are going to be applied to the link operations
         parameters = self.get_fields("parameters", [])
+
+        # tries to get the context reference that is going to be "passed"
+        # to the underlying link operation if required
+        context = self.field("context", None)
+        context = context.split(",") if context else None
+
+        # retrieves the reference to the complete set of ids of entities
+        # for which the link operation should be applied
         ids = self.field("ids", "")
         ids = ids.split(",")
+
         ids = [self.get_adapter().object_id(_id) for _id in ids if _id]
         is_global = self.field("is_global", False, cast = bool)
         model = self.get_model(model)
+
+        # creates the set of named arguments dictionary to be used
+        # for the selection of the target entities
         if ids: kwargs = dict(_id = {"$in" : ids})
         else: kwargs = dict()
+
+        # retrieves the reference to the appropriate entities (or
+        # classes) taking into account if the link is global or not
         if is_global: entities = (model,)
         else: entities = model.find_v(**kwargs)
+
+        # creates the new keyword based dictionary that is going to
+        # be used to set the named arguments for the link method call
+        # in case there's some (eg: context)
+        kwargs = dict()
+
+        # creates the dictionary that is going to hold the named
+        # arguments for the URL generation
+        if context: kwargs["context"] = context
+
+        # defines the default result value as an invalid value,
+        # this will ensure an error in the redirection process
         result = None
+
+        # iterates over the complete set of selected entities to
+        # obtain the result redirection string value for each of
+        # them, notice that only the last is going to be used
         for entity in entities:
             method = getattr(entity, link)
-            result = method(*parameters)
+            result = method(*parameters, **kwargs)
+
+        # runs the redirection of the user agent to the final
+        # result string (and URL) value
         return self.redirect(result)
 
     @appier.ensure(token = "admin")
     def operation_model(self, model, operation):
+        # ensures that the proper token is available so that the
+        # operation may be executed under the current context
+        appier.ensure_login(self, token = "admin.models." + model)
+
         # retrieves the complete set of fields that are going to
         # be used while performing the operation and runs the
         # complete set of casting operations for each of them
@@ -919,10 +1170,14 @@ class AdminPart(
 
         # in case the factory mode is enabled, a new entity has been
         # created and proper redirection must be performed so that the
-        # new entity is shown instead of the default show model
+        # new entity is shown instead of the default show model, in case
+        # the returned value from the operation is a string one the
+        # redirection is made directly to that target value
         if factory:
+            is_string = appier.legacy.is_string(result)
+            if is_string: return self.redirect(result)
             cls = result.__class__
-            model_name = cls._name()
+            model_name = cls._under()
             model_id = result._id
             return self.redirect(
                 self.url_for(
@@ -935,7 +1190,54 @@ class AdminPart(
         # runs the default redirection process that displays the model
         # page for the model used as target for the operation
         return self.redirect(
-            next or self.url_for("admin.show_model", model = model._name())
+            next or self.url_for("admin.show_model", model = model._under())
+        )
+
+    @appier.ensure(token = "admin")
+    def view_model(self, model, view):
+        appier.ensure_login(self, token = "admin.models." + model)
+
+        parameters = self.get_fields("parameters", [])
+        _id = self.field("id", None)
+        is_global = False if _id else True
+
+        model = self.get_model(model)
+        model.assert_is_concrete_g()
+
+        if _id: entity = model.get_v(_id = self.get_adapter().object_id(_id))
+        else: entity = model
+
+        object = appier.get_object(
+            alias = True,
+            page = True,
+            find = True
+        )
+
+        definition = model.view(view)
+        parameters = definition.cast(parameters)
+        method = getattr(entity, view)
+        result = method(
+            rules = False,
+            meta = True,
+            *parameters,
+            **object
+        )
+        target = result["model"]
+        entities = result["entities"]
+        page = result["page"]
+        names = result.get("names", None)
+
+        return self.template(
+            "views/show.html.tpl",
+            section = "models",
+            model = model,
+            target = target,
+            entity = entity,
+            entities = entities,
+            page = page,
+            definition = definition,
+            names = names,
+            is_global = is_global
         )
 
     @appier.ensure(token = "admin")
@@ -964,7 +1266,7 @@ class AdminPart(
             )
 
         return self.redirect(
-            self.url_for("admin.show_model", model = model._name())
+            self.url_for("admin.show_model", model = model._under())
         )
 
     @appier.ensure(token = "admin")
@@ -976,11 +1278,14 @@ class AdminPart(
             meta = True,
             _id = self.get_adapter().object_id(_id)
         )
+        previous_url, next_url = self._entity_urls(entity)
         return self.template(
             "entities/show.html.tpl",
             section = "models",
             entity = entity,
-            model = model
+            model = model,
+            previous_url = previous_url,
+            next_url = next_url
         )
 
     @appier.ensure(token = "admin")
@@ -1039,7 +1344,7 @@ class AdminPart(
         return self.redirect(
             self.url_for(
                 "admin.show_entity",
-                model = model._name(),
+                model = model._under(),
                 _id = _id
             )
         )
@@ -1053,12 +1358,12 @@ class AdminPart(
         return self.redirect(
             self.url_for(
                 "admin.show_model",
-                model = model._name()
+                model = model._under()
             )
         )
 
     def facebook(self):
-        next = self.field("next")
+        next = self.field("next", "")
         context = self.field("context", "login")
         state = context + ":" + next
         secure = not context == "login"
@@ -1087,6 +1392,13 @@ class AdminPart(
            next or self.url_for("admin.social")
         )
 
+    def unset_facebook(self):
+        next = self.field("next")
+        self.unset_facebook_account()
+        return self.redirect(
+           next or self.url_for(self.owner.admin_login_redirect)
+        )
+
     def oauth_facebook(self):
         code = self.field("code")
         state = self.field("state")
@@ -1105,7 +1417,7 @@ class AdminPart(
         )
 
     def github(self):
-        next = self.field("next")
+        next = self.field("next", "")
         context = self.field("context", "login")
         state = context + ":" + next
         secure = not context == "login"
@@ -1134,6 +1446,13 @@ class AdminPart(
            next or self.url_for("admin.social")
         )
 
+    def unset_github(self):
+        next = self.field("next")
+        self.unset_twitter_account()
+        return self.redirect(
+           next or self.url_for(self.owner.admin_login_redirect)
+        )
+
     def oauth_github(self):
         code = self.field("code")
         state = self.field("state")
@@ -1152,7 +1471,7 @@ class AdminPart(
         )
 
     def google(self):
-        next = self.field("next")
+        next = self.field("next", "")
         context = self.field("context", "login")
         state = context + ":" + next
         secure = not context == "login"
@@ -1188,6 +1507,13 @@ class AdminPart(
            next or self.url_for("admin.social")
         )
 
+    def unset_google(self):
+        next = self.field("next")
+        self.unset_google_account()
+        return self.redirect(
+           next or self.url_for(self.owner.admin_login_redirect)
+        )
+
     def oauth_google(self):
         code = self.field("code")
         state = self.field("state")
@@ -1214,7 +1540,7 @@ class AdminPart(
         )
 
     def live(self):
-        next = self.field("next")
+        next = self.field("next", "")
         context = self.field("context", "login")
         state = context + ":" + next
         secure = not context == "login"
@@ -1243,6 +1569,13 @@ class AdminPart(
            next or self.url_for("admin.social")
         )
 
+    def unset_live(self):
+        next = self.field("next")
+        self.unset_live_account()
+        return self.redirect(
+           next or self.url_for(self.owner.admin_login_redirect)
+        )
+
     def oauth_live(self):
         code = self.field("code")
         state = self.field("state")
@@ -1261,7 +1594,7 @@ class AdminPart(
         )
 
     def twitter(self):
-        next = self.field("next")
+        next = self.field("next", "")
         context = self.field("context", "login")
         state = context + ":" + next
         secure = not context == "login"
@@ -1288,6 +1621,13 @@ class AdminPart(
            next or self.url_for("admin.social")
         )
 
+    def unset_twitter(self):
+        next = self.field("next")
+        self.unset_twitter_account()
+        return self.redirect(
+           next or self.url_for(self.owner.admin_login_redirect)
+        )
+
     def oauth_twitter(self):
         oauth_verifier = self.field("oauth_verifier")
         state = self.field("state")
@@ -1350,6 +1690,12 @@ class AdminPart(
             tokens = account.tokens()
         )
 
+    @appier.ensure(token = "admin")
+    def me_account_api(self):
+        account_c = self._get_cls(self.account_c)
+        account = account_c.from_session(map = True)
+        return account
+
     def socials(self):
         socials = []
         if self.has_facebook(): socials.append("facebook")
@@ -1422,7 +1768,7 @@ class AdminPart(
     def _available(self, models):
         models = self._concrete(models)
         return [model for model in models if\
-             appier.check_login(self, "admin.models." + model._name())]
+             appier.check_login(self, "admin.models." + model._under())]
 
     def _sort(self, object, model):
         if "sort" in object: return object
@@ -1466,3 +1812,24 @@ class AdminPart(
 
     def _to_meta(self, type):
         return appier.Model._to_meta(type)
+
+    def _get_cls(self, default = None):
+        cls = self.field("cls", None)
+        if not cls: return default
+        return self.owner.get_model(cls)
+
+    def _entity_urls(self, entity):
+        model = entity.__class__
+        previous = entity.previous()
+        next = entity.next()
+        previous_url = self.url_for(
+            "admin.show_entity",
+            model = model._under(),
+            _id = previous._id
+        ) if previous else None
+        next_url = self.url_for(
+            "admin.show_entity",
+            model = model._under(),
+            _id = next._id
+        ) if next else None
+        return previous_url, next_url
