@@ -41,6 +41,9 @@ import appier
 
 class Facebook(object):
 
+    def ensure_facebook(self):
+        appier.ensure_pip("facebook", package = "facebook_api")
+
     def has_facebook(self):
         try: import facebook
         except: facebook = None
@@ -49,35 +52,45 @@ class Facebook(object):
         if not appier.conf("FB_SECRET"): return False
         return True
 
-    def ensure_facebook_account(self, create = True, owner = None):
-        owner = owner or appier.get_app()
+    def ensure_facebook_account(self, create = True, safe = True):
         api = self.get_facebook_api()
         user = api.self_user()
-        account = owner.admin_account.get(
-            email = user["email"],
+        email = user.get("email", None)
+        facebook_id = user["id"]
+        account = self.owner.admin_account.get(
+            facebook_id = facebook_id,
             rules = False,
             raise_e = False
         )
+        account = account or self.owner.admin_account.from_session()
+        account = account or email and self.owner.admin_account.get(
+            email = email,
+            rules = False,
+            raise_e = False
+        )
+
+        if safe and "fb.access_token" in self.session:
+            del self.session["fb.access_token"]
 
         if not account:
             if not create: raise appier.NotFoundError(
                 message = "No account found for facebook account"
             )
 
-            account = owner.admin_account(
-                username = user["email"],
-                email = user["email"],
+            account = self.owner.admin_account(
+                username = email,
+                email = email,
                 password = api.access_token,
                 password_confirm = api.access_token,
-                facebook_id = user["id"],
+                facebook_id = facebook_id,
                 facebook_token = api.access_token,
-                type = owner.admin_account.USER_TYPE
+                type = self.owner.admin_account.USER_TYPE
             )
             account.save()
             account = account.reload(rules = False)
 
         if not account.facebook_id:
-            account.facebook_id = user["id"]
+            account.facebook_id = facebook_id
             account.facebook_token = api.access_token
             account.save()
 
@@ -90,6 +103,12 @@ class Facebook(object):
 
         return account
 
+    def unset_facebook_account(self):
+        account = self.owner.admin_account.from_session()
+        account.facebook_id = None
+        account.facebook_token = None
+        account.save()
+
     def ensure_facebook_api(self, state = None, scope = None, refresh = False):
         access_token = self.session.get("fb.access_token", None)
         if access_token and not refresh: return
