@@ -41,6 +41,9 @@ import appier
 
 class Github(object):
 
+    def ensure_github(self):
+        appier.ensure_pip("github", package = "github_api_python")
+
     def has_github(self):
         try: import github
         except: github = None
@@ -49,35 +52,45 @@ class Github(object):
         if not appier.conf("GITHUB_SECRET"): return False
         return True
 
-    def ensure_github_account(self, create = True, owner = None):
-        owner = owner or appier.get_app()
+    def ensure_github_account(self, create = True, safe = True):
         api = self.get_github_api()
         user = api.self_user()
-        account = models.Account.get(
-            email = user["email"],
+        email = user["email"]
+        github_login = user["login"]
+        account = self.owner.admin_account.get(
+            github_login = github_login,
             rules = False,
             raise_e = False
         )
+        account = account or self.owner.admin_account.from_session()
+        account = account or self.owner.admin_account.get(
+            email = email,
+            rules = False,
+            raise_e = False
+        )
+
+        if safe and "gh.access_token" in self.session:
+            del self.session["gh.access_token"]
 
         if not account:
             if not create: raise appier.NotFoundError(
                 message = "No account found for github account"
             )
 
-            account = owner.admin_account(
-                username = user["email"],
-                email = user["email"],
+            account = self.owner.admin_account(
+                username = email,
+                email = email,
                 password = api.access_token,
                 password_confirm = api.access_token,
-                github_login = user["login"],
+                github_login = github_login,
                 github_token = api.access_token,
-                type = owner.admin_account.USER_TYPE
+                type = self.owner.admin_account.USER_TYPE
             )
             account.save()
             account = account.reload(rules = False)
 
         if not account.github_login:
-            account.github_login = user["login"]
+            account.github_login = github_login
             account.github_token = api.access_token
             account.save()
 
@@ -90,6 +103,12 @@ class Github(object):
 
         return account
 
+    def unset_github_account(self):
+        account = self.owner.admin_account.from_session()
+        account.github_login = None
+        account.github_token = None
+        account.save()
+
     def ensure_github_api(self, state = None, scope = None, refresh = False):
         access_token = self.session.get("gh.access_token", None)
         if access_token and not refresh: return
