@@ -41,6 +41,9 @@ import appier
 
 class Google(object):
 
+    def ensure_google(self):
+        appier.ensure_pip("google", package = "google_api")
+
     def has_google(self):
         try: import google
         except: google = None
@@ -49,36 +52,45 @@ class Google(object):
         if not appier.conf("GOOGLE_SECRET"): return False
         return True
 
-    def ensure_google_account(self, create = True, owner = None):
-        owner = owner or appier.get_app()
+    def ensure_google_account(self, create = True, safe = True):
         api = self.get_google_api()
         user = api.self_user()
         email = user["emails"][0]["value"]
-        account = owner.admin_account.get(
+        google_id = user["id"]
+        account = self.owner.admin_account.get(
+            google_id = google_id,
+            rules = False,
+            raise_e = False
+        )
+        account = account or self.owner.admin_account.from_session()
+        account = account or self.owner.admin_account.get(
             email = email,
             rules = False,
             raise_e = False
         )
 
+        if safe and "gg.access_token" in self.session:
+            del self.session["gg.access_token"]
+
         if not account:
             if not create: raise appier.NotFoundError(
                 message = "No account found for google account"
             )
 
-            account = owner.admin_account(
+            account = self.owner.admin_account(
                 username = email,
                 email = email,
                 password = api.access_token,
                 password_confirm = api.access_token,
-                google_id = user["id"],
+                google_id = google_id,
                 google_token = api.access_token,
-                type = owner.admin_account.USER_TYPE
+                type = self.owner.admin_account.USER_TYPE
             )
             account.save()
             account = account.reload(rules = False)
 
         if not account.google_id:
-            account.google_id = user["id"]
+            account.google_id = google_id
             account.google_token = api.access_token
             account.save()
 
@@ -91,6 +103,12 @@ class Google(object):
 
         return account
 
+    def unset_google_account(self):
+        account = self.owner.admin_account.from_session()
+        account.google_id = None
+        account.google_token = None
+        account.save()
+
     def ensure_google_api(
         self,
         state = None,
