@@ -41,6 +41,9 @@ import appier
 
 class Live(object):
 
+    def ensure_live(self):
+        appier.ensure_pip("live", package = "live_api")
+
     def has_live(self):
         try: import live
         except: live = None
@@ -49,36 +52,45 @@ class Live(object):
         if not appier.conf("LIVE_SECRET"): return False
         return True
 
-    def ensure_live_account(self, create = True, owner = None):
-        owner = owner or appier.get_app()
+    def ensure_live_account(self, create = True, safe = True):
         api = self.get_live_api()
         user = api.self_user()
         email = user["emails"]["preferred"]
-        account = owner.admin_account.get(
+        live_id = user["id"]
+        account = self.owner.admin_account.get(
+            live_id = live_id,
+            rules = False,
+            raise_e = False
+        )
+        account = account or self.owner.admin_account.from_session()
+        account = account or self.owner.admin_account.get(
             email = email,
             rules = False,
             raise_e = False
         )
 
+        if safe and "live.access_token" in self.session:
+            del self.session["live.access_token"]
+
         if not account:
             if not create: raise appier.NotFoundError(
                 message = "No account found for live account"
             )
 
-            account = owner.admin_account(
+            account = self.owner.admin_account(
                 username = email,
                 email = email,
                 password = api.access_token,
                 password_confirm = api.access_token,
-                live_id = user["id"],
+                live_id = live_id,
                 live_token = api.access_token,
-                type = owner.admin_account.USER_TYPE
+                type = self.owner.admin_account.USER_TYPE
             )
             account.save()
             account = account.reload(rules = False)
 
         if not account.live_id:
-            account.live_id = user["id"]
+            account.live_id = live_id
             account.live_token = api.access_token
             account.save()
 
@@ -91,6 +103,12 @@ class Live(object):
 
         return account
 
+    def unset_live_account(self):
+        account = self.owner.admin_account.from_session()
+        account.live_id = None
+        account.live_token = None
+        account.save()
+
     def ensure_live_api(self, state = None, scope = None, refresh = False):
         access_token = self.session.get("live.access_token", None)
         if access_token and not refresh: return
