@@ -41,6 +41,9 @@ import appier
 
 class Twitter(object):
 
+    def ensure_twitter(self):
+        appier.ensure_pip("twitter", package = "twitter_api")
+
     def has_twitter(self):
         try: import twitter
         except: twitter = None
@@ -49,36 +52,49 @@ class Twitter(object):
         if not appier.conf("TWITTER_SECRET"): return False
         return True
 
-    def ensure_twitter_account(self, create = True, owner = None):
-        owner = owner or appier.get_app()
+    def ensure_twitter_account(self, create = True, safe = True):
         api = self.get_twitter_api()
         user = api.verify_account()
         email = "%s@twitter.com" % user["screen_name"]
-        account = owner.admin_account.get(
+        twitter_username = user["screen_name"]
+        account = self.owner.admin_account.get(
+            twitter_username = twitter_username,
+            rules = False,
+            raise_e = False
+        )
+        account = account or self.owner.admin_account.from_session()
+        account = account or self.owner.admin_account.get(
             email = email,
             rules = False,
             raise_e = False
         )
 
+        if safe and "tw.oauth_token" in self.session:
+            del self.session["tw.oauth_token"]
+        if safe and "tw.oauth_token_secret" in self.session:
+            del self.session["tw.oauth_token_secret"]
+        if safe and "tw.oauth_temporary" in self.session:
+            del self.session["tw.oauth_temporary"]
+
         if not account:
             if not create: raise appier.NotFoundError(
                 message = "No account found for twitter account"
             )
 
-            account = owner.admin_account(
-                username = user["screen_name"],
+            account = self.owner.admin_account(
+                username = twitter_username,
                 email = email,
                 password = api.oauth_token,
                 password_confirm = api.oauth_token,
-                twitter_username = user["screen_name"],
+                twitter_username = twitter_username,
                 twitter_token = api.oauth_token,
-                type = owner.admin_account.USER_TYPE
+                type = self.owner.admin_account.USER_TYPE
             )
             account.save()
             account = account.reload(rules = False)
 
         if not account.twitter_username:
-            account.twitter_username = user["screen_name"]
+            account.twitter_username = twitter_username
             account.twitter_token = api.oauth_token
             account.save()
 
@@ -91,6 +107,12 @@ class Twitter(object):
 
         return account
 
+    def unset_twitter_account(self):
+        account = self.owner.admin_account.from_session()
+        account.twitter_username = None
+        account.twitter_token = None
+        account.save()
+
     def ensure_twitter_api(self, state = None, refresh = False):
         oauth_token = self.session.get("tw.oauth_token", None)
         oauth_token_secret = self.session.get("tw.oauth_token_secret", None)
