@@ -91,7 +91,7 @@ class OpbeatPart(appier.Part):
             exception.message or str(exception)
 
         lines = traceback.format_exc().splitlines()
-        lines = self._lines(lines)
+        lines = appier.App._lines(lines)
 
         payload = dict(
             message = message,
